/*
 *
 * Simple class to output RAPID code from a set of poses
 *
 */
class RapidWriter {
  
  RapidWriter() {
    startAtHome = false;
  }
  
  /* Constuctor - specify if we should start at the robot home location first */
  RapidWriter(boolean startAtHome) {
    this.startAtHome = startAtHome;
  }
  
  PVector toolSize;
  float q1, q2, q3, q4; // quaternion

  // Locations & Orientations to move to
  ArrayList<PVector>locs = new ArrayList<PVector>();
  ArrayList<PVector>rots = new ArrayList<PVector>();  
  
  PrintWriter output;
  
  boolean startAtHome = false;
  
  /* Set the tool data */
  void setToolData(PVector toolSize, float q1, float q2, float q3, float q4) {
    this.toolSize = toolSize;
    this.q1 = q1;
    this.q2 = q2;
    this.q3 = q3;
    this.q4 = q4;    
  }

  /* Add a MoveJ command */  
  void addMoveJ(PVector loc, PVector rot) {
    locs.add(loc);
    rots.add(rot);
  }
  
  /* Write the RAPID program out */
  void write(String filename) {
    
    // Create the Module file
    output = createWriter(filename + "/" + "Module1.mod"); 
     
    // Copy the program file 
    String src = dataPath("Program.pgf");
    String dst = sketchPath(filename + "/" + filename + ".pgf");   
    try {
      copyFile(new File(src), new File(dst));
    } catch (IOException ex) {
      println("Error copying file");
      return;
    }
   
    output.println("MODULE Module1");
    output.println();
    output.println("  ! Created with RAPID_P5 for FUTURE CNC");
    output.println("  PERS tooldata MyTool:=[TRUE,[[" +
                      toolSize.x + "," + toolSize.y + "," + toolSize.z + "],[" +
                      q1 + "," + q2 + "," + q3 + "," + q4 + "]],[1,[0,0,1],[1,0,0,0],0,0,0]];");
    if(startAtHome) {
      output.println("  CONST jointtarget homePose := [[0,0,0,0,0,0],[0,0,0,0,0,0]];");                  
    }
    
    output.println();
    output.println("FUNC robtarget getRobtarget(num x, num y, num z, num eulerX, num eulerY, num eulerZ)");
    output.println("  VAR robtarget targetToMoveTo:= [ [0,0,0], [1, 0, 0, 0], [0, 0, 0, 0], [ 9E9, 9E9, 9E9, 9E9, 9E9, 9E9] ];");
    output.println("  VAR jointtarget testJoint;");
    output.println("  VAR robtarget testTarget;");
    output.println("  targetToMoveTo.trans := [x,y,z];");
    output.println("  targetToMoveTo.rot := OrientZYX(eulerZ, eulerY, eulerX);");
    output.println("  testJoint  := CJointT();");
    output.println("  testTarget := CalcRobT(testJoint, MyTool);");
    output.println("  targetToMoveTo.robconf := testTarget.robconf;");
    output.println("  RETURN targetToMoveTo;");
    output.println("ENDFUNC");
    output.println();
    
    output.println("PROC main()");
    output.println("  ConfL\\Off;");
    output.println("  ConfJ\\Off;");
    
    if(startAtHome) {
      output.println();
      output.println("  MoveAbsJ homePose, v100, z1, MyTool;");
      output.println();
    }

    for(int i=0; i< locs.size(); i++) {
      PVector loc = locs.get(i);
      PVector rot = rots.get(i);      
      output.println("  MoveJ getRobtarget(" + 
                      loc.x + "," + loc.y + "," + loc.z + "," + 
                      rot.x + "," + rot.y + "," + rot.z +"), v100, z1, MyTool;");
    }
    
    output.println("ENDPROC");    
    output.println();
    output.println("ENDMODULE"); 
 
    output.flush(); 
    output.close(); 
    
    println("Rapid Program written to: " + sketchPath(filename));
    
  }
  
  // Copies src file to dst file.
  // If the dst file does not exist, it is created
  void copyFile(File src, File dst) throws IOException {
      InputStream in = new FileInputStream(src);
      OutputStream out = new FileOutputStream(dst);
  
      // Transfer bytes from in to out
      byte[] buf = new byte[1024];
      int len;
      while ((len = in.read(buf)) > 0) {
          out.write(buf, 0, len);
      }
      in.close();
      out.close();
  }
  
}
